<?php
/*
Plugin Name: GenieWords
Plugin URI: https://plugins.geniewords.com/geniewords.zip
Description: The plugin which is going to make sure your Geniewords account is in sync.
Author: GenieWords
Text Domain: geniewords
Domain Path: /languages/
Version: 1.4.16
Author URI: https://geniewords.com
*/

if (!defined('GENIEWORDS_ADMIN_DIR')) {
    define('GENIEWORDS_ADMIN_DIR', plugin_dir_path(__FILE__));
}

if (!defined('GENIEWORDS_FILE')) {
    define('GENIEWORDS_FILE', __FILE__);
}

if (!defined('GENIEWORDS_USER_ID')) {
    $get_user_id = get_post_meta(1, 'geniewords_userid');
    if (!empty($get_user_id[0])) {
        define('GENIEWORDS_USER_ID', $get_user_id[0]);
    } else {
        define('GENIEWORDS_USER_ID', '');
    }
}

if (!defined('GENIE_MERCHANT_ID')) {
    $get_merchantId = get_post_meta(1, 'geniewords_merchantid');
    if (!empty($get_merchantId[0])) {
        define('GENIE_MERCHANT_ID', $get_merchantId[0]);
    } else {
        define('GENIE_MERCHANT_ID', '');
    }
}

if (!defined('GENIEWORDS_FB_NAME')) {
    define('GENIEWORDS_FB_NAME', 'Geniewords testing');
}

if(!function_exists('request_filesystem_credentials')){
    require_once ABSPATH  . 'wp-admin/includes/file.php';
}

require_once ABSPATH  . 'wp-admin/includes/plugin-install.php';
require_once ABSPATH  . 'wp-admin/includes/plugin.php';
require_once ABSPATH  . 'wp-admin/includes/misc.php';
require_once ABSPATH . 'wp-includes/pluggable.php';
require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';

if (!class_exists('Geniewords_js')) {
    class Geniewords_js
    { 
        protected static $_genieWordApiUrl = 'https://api.geniewords.com';
        protected static $_websiteUrl = '';

        // Constructor
        function __construct()
        {
            $current_plugin_info = get_plugin_data(plugin_dir_path(__FILE__).'geniewords.php');
            $wordpress_org_version = plugins_api( 'plugin_information', array( 'slug' => 'geniewords' ) );

            add_action('wp_head', array($this, 'add_fb_pixel_code_in_head'));

            if(version_compare($wordpress_org_version->version, $current_plugin_info['Version']) === 1) {
                update_post_meta(1, 'new_version_available', 1);
            } else {
                update_post_meta(1, 'new_version_available', 0);
            }

            add_filter( 'auto_update_plugin', array($this, 'auto_update_specific_plugins'), 10, 2 );

            ini_set('error_log', plugin_dir_path(__FILE__) . '/debug.log');
            self::$_websiteUrl = site_url('/');

            register_deactivation_hook(GENIEWORDS_FILE, array($this, 'deactive_javascript_code'));

            add_action('admin_menu', array($this, 'add_menu_left'));
            add_action('activated_plugin', array($this, 'geniewords_activation'));
            add_action('wp_head', array($this, 'add_meta_tags'));

            add_action('wp_ajax_update_add_analytics_to_head', array($this, 'update_add_analytics_to_head'), 10, 1);
            add_action('wp_ajax_update_analytics_custom_value', array($this, 'update_analytics_custom_value'), 10, 1);
            add_action('wp_ajax_update_plugin_version', array($this, 'update_plugin_version'), 10, 1);
            add_action('wp_ajax_update_pixel_custom_value', array($this, 'update_pixel_custom_value'), 10, 1);
            add_action('wp_ajax_update_add_facebook_pixel_to_head', array($this, 'update_add_facebook_pixel_to_head'), 10, 1);

            // FB EVENTS
            add_action('woocommerce_add_to_cart', array($this, 'facebook_pixel_track_add_to_cart'));
            add_action('wp_head', array($this, 'facebook_pixel_add_tracking_code_in_head'));
            add_action('wp_head', array($this, 'facebook_pixel_track_payment_complete'));
            add_action('wp_head', array($this, 'facebook_pixel_track_search_and_view_content'), 10, 1);

            add_action('wp_ajax_syncronise_action', array($this, 'get_woocommerce_products'), 10, 1);
            add_action('wp_ajax_fb_syncronise_action', array($this, 'add_woocommerce_fb_products'), 10, 1);
            add_action('wp_ajax_signout_action', array($this, 'signout_users'));
            add_action('admin_init', array($this, 'plugin_translation_setup'));
            add_action('woocommerce_update_product', array($this, 'update_product_hook_function'), 10, 1);
            add_action('wp_trash_post', array($this, 'delete_product_hook_function'), 10, 1);
            add_action('publish_product', array($this, 'add_product_hook_function'), 10, 1);

            add_action('wp_head', array($this, 'put_facebook_metatag_to_head'));
            add_action('wp_head', array($this, 'put_google_shopping_metatag_to_head'));
            add_action('wp_head', array($this, 'add_analytics_in_head'));

            add_action('admin_init', array($this, 'enableDisableVariantsCheckbox'));

            if(isset($_POST['genieSubmit']) || isset($_POST['genieReSubmit'])) {
                if (!empty($_POST['email-geniewords']) && !empty($_POST['password-geniewords'])) {
                    $get_response_data = Geniewords_js::get_idToken(1, $_POST['email-geniewords'], $_POST['password-geniewords']);
                } else if(isset($_POST['genieReSubmit'])) {
                    $user_email = get_post_meta(1, 'geniewords_user_email');
                    $user_email = current($user_email);
                    $user_password = get_post_meta(1, 'geniewords_user_password');
                    $user_password = current($user_password);
                    $get_response_data = Geniewords_js::get_idToken(1, $user_email, $user_password);
                }

                if (!$get_response_data) {
                    $errorMsg = __('Please enter correct e-mail & password', 'geniewords');
                    $this->setTransientErrorMsg($errorMsg);
                } else {
                    if(isset($_POST['genieSubmit'])) {
                        $user_email = get_post_meta(1, 'geniewords_user_email');
                        $user_password = get_post_meta(1, 'geniewords_user_password');

                        if (empty($user_email[0])) {
                            update_post_meta(1, 'geniewords_user_email', sanitize_text_field($_POST['email-geniewords']));
                        } else {
                            update_post_meta(1, 'geniewords_user_email', sanitize_text_field($_POST['email-geniewords']));
                        }

                        if (empty($user_password[0])) {
                            update_post_meta(1, 'geniewords_user_password', sanitize_text_field($_POST['password-geniewords']));
                        } else {
                            update_post_meta(1, 'geniewords_user_password', sanitize_text_field($_POST['password-geniewords']));
                        }
                    }

                    $user_id = $get_response_data->localId;
                    $this->initializeData($user_id);
                }
            }

            $get_user_id = get_post_meta(1, 'geniewords_userid');
            if(!empty(GENIEWORDS_USER_ID)) {
                $this->initializeData($get_user_id[0]);
            }
        }

        function facebook_pixel_track_add_to_cart() {
            global $woocommerce;

            $product_ids = [];
            $content = array();
            $count = 0;

            $total_price = 0;
            $currency = 'DKK';
            foreach ($woocommerce->cart->cart_contents as $cart_content) {
                $product_id = (string) $cart_content['product_id'];

                $wc_product = new WC_Product($cart_content['product_id']);

                if (!empty($wc_product->regular_price)) {
                    $price = $wc_product->regular_price;
                } else if (!empty($wc_product->price)) {
                    $price = $wc_product->price;
                }

                if (!empty($wc_product->sale_price)) {
                    $price = (int)$wc_product->sale_price;
                }

                $currentIndex = (string) $count++;
                $content[$currentIndex] = array(
                    "id" => $product_id,
                    "quantity" => 1
                );

                if(isset($price)) {
                    $total_price+=($price * $cart_content['quantity']);
                }

                if(isset($cart_content['currency'])) {
                    $currency = $cart_content['currency'];
                }

                array_push($product_ids, $product_id);
            }

            $fb_track_data = array(
                "content_ids" => $product_ids,
                "content_type" => 'product',
                "value" =>$total_price,
                "currency" => $currency
            );

            update_post_meta(1, 'fb_track_data', $fb_track_data);
        }

        function facebook_pixel_add_tracking_code_in_head() {
            $fb_track_data = get_post_meta(1, 'fb_track_data');

            delete_post_meta(1, 'fb_track_data'); // after first use delete this meta not to make duplicate event tracking

            if(isset($fb_track_data[0])) {
                ?>
                <script>
                    fbq('track', 'AddToCart', <?php echo json_encode($fb_track_data[0]) ?>);
                </script>
                <!-- End Facebook Pixel Code -->
                <?php
            }
        }

        function facebook_pixel_track_payment_complete( ) {
            if ( function_exists('is_checkout') && is_checkout() && !empty( is_wc_endpoint_url('order-received') ) ) {
                global $wp;
                $url_parse = home_url( add_query_arg( array(), $wp->request ) );

                $order_id = preg_split("#/#", parse_url($url_parse)['path'])[3];

                $order = wc_get_order($order_id);

                $items = $order->get_items();

                $product_ids = [];
                $content = array();
                $count = 0;

                foreach ($items as $item) {
                    $product_id = (string) $item['product_id'];

                    $content[$count++] = array(
                        "id" => $product_id,
                        "quantity" => 1
                    );

                    array_push($product_ids, $product_id);
                }

                $currency = 'DKK';
                if(isset($order->currency)) {
                    $currency = $order->currency;
                }

                $old_order_id = get_post_meta(1, 'order_completed_data');
                if(!isset($old_order_id[0]) || $old_order_id[0] !== $order_id) {
                    $fb_track_data = array(
                        "content_ids" => $product_ids,
                        "contents" => $content,
                        "value" => $order->total,
                        "currency" => $currency,
                        "content_type" => "product"
                    );

                    update_post_meta(1, 'order_completed_data', $order_id);
                    ?>
                    <script>
                        fbq('track', 'Purchase', <?php echo json_encode($fb_track_data) ?>);
                    </script>
                    <?php
                }
            }
        }

        function facebook_pixel_track_search_and_view_content() {
            ?>
            <?php if( function_exists('is_product_category') && is_product_category() ) {
                global $wp_query;
                $id = $wp_query->get_queried_object()->term_id;

                $category = get_term_by( 'id', $id, 'product_cat' );

                $query = new WP_Query( array(
                    'post_type'      => 'product',
                    'category' => array($category->slug),
                ) );

                $product_ids = [];
                $content = array();
                $count = 0;

                while ( $query->have_posts() ) : $query->the_post();
                    $product_id = (string) get_the_id();

                    $content[$count++] = array(
                            "id" => $product_id,
                            "quantity" => 1
                    );

                    array_push($product_ids, $product_id);
                endwhile;

                $fb_track_data = array(
                    "content_ids" => $product_ids,
                    "content_type" => 'product',
                    "content_category" => $category->name,
                    "source" => 'woocommerce',
                    "content_name" => $category->name,
                    "contents" => $content
                );
                ?>
                <script>
                    fbq('track', 'ViewCategory', <?php echo json_encode($fb_track_data) ?>);
                </script> <?php
            }

            if( function_exists('is_product') && is_product() ) {
                $id = get_the_ID();

                $wc_product = new WC_Product($id);

                if (!empty($wc_product->regular_price)) {
                    $price = $wc_product->regular_price;
                } else if (!empty($wc_product->price)) {
                    $price = $wc_product->price;
                }

                if (!empty($wc_product->sale_price)) {
                    $price = (int)$wc_product->sale_price;
                }

                $fb_track_data = array(
                    "content_ids" => [ (string) $id ],
                    "content_type" => 'product'
                );

                if(isset($price)) {
                    $fb_track_data['value'] = $price;
                }
                ?>
                <script>
                    fbq('track', 'ViewContent', <?php echo json_encode($fb_track_data) ?>);
                </script>
                <?php
            }
        }

        function update_plugin_version() {
            $installer = new WP_Upgrader();

            $wordpress_org_version = plugins_api( 'plugin_information', array( 'slug' => 'geniewords' ) );

            $installer->run(array('package' => 'https://downloads.wordpress.org/plugin/geniewords.'.$wordpress_org_version->version.'.zip', 'destination' => WP_PLUGIN_DIR, 'abort_if_destination_exists' => false));

            update_post_meta(1, 'new_version_available', 0);
        }

        function auto_update_specific_plugins ( $update, $item ) {
            $plugins = array (
                'geniewords'
            );

            if ( in_array( $item->slug, $plugins ) ) {
                return true;
            }
        }

        function initializeData($user_id) {
            if (!defined('GENIEWORDS_ID_TOKEN')) {
                $id_token = Geniewords_js::get_idToken();
            } else {
                $id_token = GENIEWORDS_ID_TOKEN;
            }

            $args_get = array(
                'method' => 'GET',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    'Authorization' => 'Bearer ' . $id_token
                )
            );

            $basicinfo_url = self::$_genieWordApiUrl . '/users/' . $user_id . '/basicInfo';
            $response_get = wp_remote_get($basicinfo_url, $args_get);
            $httpResponse = wp_remote_retrieve_response_code($response_get);

            //Check if user info api gives does not give success result.
            if ($httpResponse === 200) {
                $transMsg['basicInfo'] = 1;
                $this->setGenieTrans($transMsg);

                $get_refresh_token = $response_get['body'];
                $get_refresh_token = json_decode($get_refresh_token);

                $this->setPostMetaFieldValuesForFb($user_id, $get_refresh_token->company, null, null, null, null, null);

                $get_user_is_logedin = get_post_meta(1, 'geniewords_user_loggedin');
                if (empty($get_user_is_logedin[0])) {
                    add_post_meta(1, 'geniewords_user_loggedin', 1);
                } else if (!empty($get_user_is_logedin)) {
                    update_post_meta(1, 'geniewords_user_loggedin', 1);
                }

                if (isset($get_refresh_token->analyticsViewId)) {
                    update_post_meta(1, 'geniewords_user_has_analytics', 1);
                } else {
                    update_post_meta(1, 'geniewords_user_has_analytics', 0);
                }

                if (isset($get_refresh_token->facebookAuthInfo)
                    && isset($get_refresh_token->facebookAuthInfo->pixelInfo)) {
                    update_post_meta(1, 'geniewords_user_has_pixel', 1);
                } else {
                    update_post_meta(1, 'geniewords_user_has_pixel', 0);
                }

                if ($get_refresh_token && $get_refresh_token->googleShoppingMetaTag) {
                    $googleShoppingMetaTag = str_replace("\\", '', $get_refresh_token->googleShoppingMetaTag);
                    update_post_meta(1, 'geniewords_google_shopping_meta_tag', $googleShoppingMetaTag);
                }

                if ($get_refresh_token && $get_refresh_token->facebookData && $get_refresh_token->facebookData->facebookMetaTag) {
                    $facebookMetaTag = str_replace("\\", '', $get_refresh_token->facebookData->facebookMetaTag);
                    update_post_meta(1, 'geniewords_facebook_meta_tag', $facebookMetaTag);
                }

                if(in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
                    foreach ($get_refresh_token->products as $get_title) {
                        if ($get_title->title == 'Facebook Retargeting') {
                            $transMsg['facebookHeading'] = 1;
                            $this->setGenieTransFacebook($transMsg);
                            $company_name = $get_refresh_token->company;
                            $fb_short_token = isset($get_refresh_token->facebookAuthInfo->shortLiveToken) ? $get_refresh_token->facebookAuthInfo->shortLiveToken : '';
                            $fb_longlive_token = isset($get_refresh_token->facebookAuthInfo->longLiveToken) ? $get_refresh_token->facebookAuthInfo->longLiveToken : '';
                            $fb_business_id = isset($get_refresh_token->facebookAuthInfo->adAccount->business_id) ? $get_refresh_token->facebookAuthInfo->adAccount->business_id : '';
                            $fb_pixelinfo_code = isset($get_refresh_token->facebookAuthInfo->pixelInfo->code) ? htmlentities($get_refresh_token->facebookAuthInfo->pixelInfo->code) : '';
                            $fb_user_business_id = isset($get_refresh_token->facebookAuthInfo->facebookPage->id) ? $get_refresh_token->facebookAuthInfo->facebookPage->id : '';

                            $this->setPostMetaFieldValuesForFb($user_id, $company_name, $fb_short_token, $fb_longlive_token, $fb_business_id, $fb_pixelinfo_code, $fb_user_business_id);

                            $get_fb_user_available = get_post_meta(1, 'geniewords_fb_user_available');
                            if ($get_fb_user_available[0] !== '0' && empty($get_fb_user_available[0])) {
                                add_post_meta(1, 'geniewords_fb_user_available', 1);
                            } else if (!empty($get_fb_user_available)) {
                                update_post_meta(1, 'geniewords_fb_user_available', 1);
                            }
                        }

                        if ($get_title->title == 'Google Shopping') {
                            $transMsg['googleHeading'] = 1;
                            $this->setGenieTransGoogle($transMsg);
                            if (!empty($get_refresh_token) && isset($get_refresh_token->googleShoppingAuthData)) {
                                $refresh_token = $get_refresh_token->googleShoppingAuthData->refreshToken;
                                $merchantId = $get_refresh_token->selectedMerchant->merchantId;
                                $company_name = $get_refresh_token->company;
                                $shopping_email = $get_refresh_token->googleShoppingAuthData->googleAuthEmail;
                                $this->setPostMetaFieldValuesForGoogle($user_id, $refresh_token, $merchantId, $company_name, $shopping_email);

                                $get_gs_user_available = get_post_meta(1, 'geniewords_gs_user_available');
                                if (empty($get_gs_user_available[0])) {
                                    update_post_meta(1, 'geniewords_gs_user_available', 1);
                                } else if (!empty($get_gs_user_available)) {
                                    update_post_meta(1, 'geniewords_gs_user_available', 1);
                                }
                            } else {
                                $transMsg['googleNotSetup'] = 1;
                                $this->setGenieTransGoogle($transMsg);
                                set_transient('error_results', 'Google account was not setup with these details, sync was not completed.', 60);
                            }
                        }
                    }
                } else {
                    update_post_meta(1, 'geniewords_fb_user_available', 0);
                    update_post_meta(1, 'geniewords_gs_user_available', 0);
                }

                set_transient('success_results', 'Successfully Signin with Geniewords', 60);
            } else {
                $transMsg['basicInfo'] = 0;
                $this->setGenieTrans($transMsg);

                $errorMsg = __('Error occurred in getting user basic info.', 'geniewords');
                $this->setTransientErrorMsg($errorMsg);
            }
        }

        // After active plugin function
        function geniewords_activation($plugin)
        {
            $file = plugin_dir_path(__FILE__) . '/debug.log';
            $fp = fopen($file, 'w');
            fclose($fp);
            chmod($file, 0777);
            if ($plugin == plugin_basename(__FILE__)) {
                exit(wp_redirect(admin_url('admin.php?page=geniewords-menu')));
            }
        }

        function add_menu_left()
        {
            add_menu_page('GenieWords', 'GenieWords', 'manage_options', 'geniewords-menu', array($this, "geniewords_signin_admin_details"), plugins_url('geniewords/assest/images/favicon-gray.png'), 56);

        }

        function plugin_translation_setup()
        {
            load_plugin_textdomain('geniewords', false, dirname(plugin_basename(__FILE__)) . '/languages/');
        }

        // Include Admin File
        function geniewords_signin_admin_details()
        {
            require_once(GENIEWORDS_ADMIN_DIR . '/geniewords_signin_admin.php');
        }

        function geniewords_facebook_admin_details()
        {
            require_once(GENIEWORDS_ADMIN_DIR . '/geniewords_facebook_admin.php');
        }

        public function setTransitent($transient)
        {
            $plugin_check = $transient->checked['geniewords/geniewords.php'];
            $get_version = wp_remote_get('https://plugins.geniewords.com/live/info.json', array('timeout' => 120, 'httpversion' => '1.1'));
            $httpResponse = wp_remote_retrieve_response_code($get_version);
            if ($httpResponse == 200) {
                $get_plugin_update_data = $get_version['body'];
                // make sure that we received the data in the response is not empty
                if (empty($get_plugin_update_data)) {
                    return $transient;
                }

                //check server version against current installed version
                if ($remote = json_decode($get_plugin_update_data)) {
                    if (version_compare($plugin_check, $remote->version, '<')) {
                        $res = new stdClass();
                        $res->plugin = 'geniewords/geniewords.php';
                        $res->new_version = $remote->version;
                        $res->tested = $remote->tested;
                        $res->package = $remote->download_url;
                        $transient->response[$res->plugin] = $res;
                    }
                }
                return $transient;
            }
        }

        function get_woocommerce_products()
        {
            if (!defined('GENIEWORDS_ID_TOKEN')) {
                $id_token = Geniewords_js::get_idToken();
            } else {
                $id_token = GENIEWORDS_ID_TOKEN;
            }

            if ($id_token) {
                $get_user_id = get_post_meta(1, 'geniewords_userid');
                $args = array(
                    'method' => 'GET',
                    'timeout' => 120,
                    'httpversion' => '1.1',
                    'headers' => array(
                        'Authorization' => 'Bearer ' . $id_token
                    )
                );

                // EXCHANGE ACCESS TOKEN for GOOGLE MERCHANT CENTER
                $response_post = wp_remote_get(self::$_genieWordApiUrl . '/users/' . $get_user_id[0] . '/exchangeAccessToken?service=GOOGLE_MERCHANT_CENTER', $args);
                $httpResponse = wp_remote_retrieve_response_code($response_post);

                if ($httpResponse !== 200) {
                    $transMsg['tokenRevoked'] = 1;
                    $this->setGenieTransGoogle($transMsg);
                    $errorMsg = __("Google Token has been expired or revoked.", "geniewords");
                    $this->setTransientErrorMsg($errorMsg);
                } else {
                    $transMsg['tokenRevoked'] = 0;
                    $this->setGenieTransGoogle($transMsg);
                    $getExchangeAccessTokenData = json_decode($response_post['body']);
                    if (!defined('GENIE_ACCESS_TOKEN')) {
                        define('GENIE_ACCESS_TOKEN', $getExchangeAccessTokenData->access_token);
                    }

                    // RETRIEVE BASIC INFORMATION OF GENIEWORDS CUSTOMER
                    $args_get = array(
                        'method' => 'GET',
                        'timeout' => 120,
                        'httpversion' => '1.1',
                        'headers' => array(
                            'Authorization' => 'Bearer ' . $id_token
                        )
                    );

                    $basicinfo_url = self::$_genieWordApiUrl . '/users/' . $get_user_id[0] . '/basicInfo';
                    $response_get = wp_remote_get($basicinfo_url, $args_get);
                    $httpResponse = wp_remote_retrieve_response_code($response_get);

                    if ($httpResponse !== 200) {
                        $errorMsg = __('Error occurred in getting user basic info.', "geniewords");
                        $this->setTransientErrorMsg($errorMsg);
                    } else {
                        $get_refresh_token = json_decode($response_get['body']);

                        $googleShoppingMetaTag = $_POST['googleShoppingMetaTag'];

                        if($googleShoppingMetaTag && isset($googleShoppingMetaTag)) {
                            $data_to_be_updated = array('googleShoppingMetaTag' => $googleShoppingMetaTag);

                            $args_website = array(
                                'method' => 'PUT',
                                'timeout' => 120,
                                'httpversion' => '1.1',
                                'headers' => array(
                                    'Content-Type' => 'application/json; charset=utf-8',
                                    'Authorization' => 'Bearer ' . $id_token
                                ),
                                'body' => json_encode($data_to_be_updated)
                            );

                            $updateUserUrl = self::$_genieWordApiUrl . '/users/' . $get_user_id[0];

                            $response_post = wp_remote_post($updateUserUrl, $args_website);
                            $httpResponse = wp_remote_retrieve_response_code($response_post);

                            if ($httpResponse !== 200) {
                                echo "<pre>";
                                print_r('Error while updating meta tag for google shopping.');
                            }
                        }

                        if (isset($get_refresh_token->selectedMerchant) && isset($get_refresh_token->selectedMerchant->merchantId)) {
                            $merchantId = $get_refresh_token->selectedMerchant->merchantId;

                            $datafeed_exists = $this->create_datafeed($merchantId);
                            if($datafeed_exists) {
                                // Update feed file
                                $this->create_datafeed_xml_file($get_refresh_token->company, $get_refresh_token->website, 'GOOGLE_SHOPPING');
                            } else {
                                // 1. read basic info of user
                                // 2. retrieve access token
                                // 3. update website URL in merchant center
                                // 4. Gets a verification token to verify ownership
                                // 5. Verifies ownership of a website or domain
                                // 6. Create xml file locally
                                // 7. Creates feed

                                $transMsg['googleMerchantSelected'] = 1;
                                $this->setGenieTransGoogle($transMsg);
                                $website_url = array('websiteUrl' => self::$_websiteUrl);
                                $args_website = array(
                                    'method' => 'PUT',
                                    'timeout' => 120,
                                    'httpversion' => '1.1',
                                    'headers' => array(
                                        'Content-Type' => 'application/json; charset=utf-8',
                                        'Authorization' => 'Bearer ' . $id_token
                                    ),
                                    'body' => json_encode($website_url)
                                );
                                $merchantUrl = self::$_genieWordApiUrl . '/merchant/' . $get_user_id[0];
                                $response_post = wp_remote_post($merchantUrl, $args_website);
                                $httpResponse = wp_remote_retrieve_response_code($response_post);

                                if ($httpResponse !== 200) {
                                    $transMsg['googleMerchantFound'] = 0;
                                    $this->setGenieTransGoogle($transMsg);
                                    $errorMsg = json_decode($response_post['body'], true)['message'];
                                    $this->setTransientErrorMsg($errorMsg);
                                } else {
                                    $transMsg['googleMerchantFound'] = 1;
                                    $this->setGenieTransGoogle($transMsg);
                                    $get_response_data = json_decode($response_post['body']);

                                    if (isset($get_response_data->success) && $get_response_data->success == 1) {
                                        $website_get_url = array('site' => array('type' => 'SITE', 'identifier' => self::$_websiteUrl, 'verificationMethod' => 'META'));
                                        $args_website = array(
                                            'method' => 'POST',
                                            'timeout' => 120,
                                            'httpversion' => '1.1',
                                            'headers' => array('Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $getExchangeAccessTokenData->access_token),
                                            'body' => json_encode($website_get_url)
                                        );
                                        $response_post = wp_remote_post('https://www.googleapis.com/siteVerification/v1/token', $args_website);
                                        $httpResponse = wp_remote_retrieve_response_code($response_post);

                                        if ($httpResponse !== 200) {
                                            $transMsg['googleSiteVerificationToken'] = 0;
                                            $this->setGenieTransGoogle($transMsg);
                                            $errorMsg = __("Error occurred in generating token at google.", 'geniewords');
                                            $this->setTransientErrorMsg($errorMsg);
                                        } else {
                                            $transMsg['googleSiteVerificationToken'] = 1;
                                            $this->setGenieTransGoogle($transMsg);
                                            $get_meta_response = json_decode($response_post['body'], true);
                                            $meta_data_response = htmlentities($get_meta_response['token']);
                                            delete_post_meta(1, 'geniewords_user_web_meta');
                                            add_post_meta(1, 'geniewords_user_web_meta', $meta_data_response);
                                            $get_geniewords_shopping_email = get_post_meta(1, 'geniewords_shopping_email');

                                            //Add Google Meta tag in header.
                                            add_action('wp_head', array($this, 'add_meta_tags'));

                                            $website_verification_url = array(
                                                'site' => array('type' => 'SITE', 'identifier' => self::$_websiteUrl), 'verificationMethod' => 'META'
                                            );
                                            $args_website = array(
                                                'method' => 'POST',
                                                'timeout' => 120,
                                                'httpversion' => '1.1',
                                                'headers' => array(
                                                    "Authorization" => "Bearer " . $getExchangeAccessTokenData->access_token,
                                                    "Content-Type" => "application/json"
                                                ),
                                                'body' => json_encode($website_verification_url)
                                            );
                                            $response_post = wp_remote_post('https://www.googleapis.com/siteVerification/v1/webResource?verificationMethod=META', $args_website);
                                            $httpResponse = wp_remote_retrieve_response_code($response_post);

                                            if ($httpResponse == 200) {
                                                $transMsg['googleMetaInfoFound'] = 1;
                                                $this->setGenieTransGoogle($transMsg);
                                                $get_merchantId = get_post_meta(1, 'geniewords_merchantid');
                                                $merchantId = (int)$get_merchantId[0];

                                                $transMsg['googleSyncPassed'] = 1;
                                                $this->setGenieTransGoogle($transMsg);

                                                $this->create_datafeed($merchantId);

                                                $this->create_datafeed_xml_file($get_refresh_token->company, $get_refresh_token->website, 'GOOGLE_SHOPPING');
                                            } else {
                                                $transMsg['googleMetaInfoFound'] = 0;
                                                $this->setGenieTransGoogle($transMsg);
                                                $errorMsg = __('Required Meta information was not found in your website. Please add following meta tag in your theme header.', 'geniewords');
                                                $this->setTransientErrorMsg($errorMsg);
                                            }
                                        }
                                    }
                                }
                            }
                        } else {
                            $transMsg['googleMerchantSelected'] = 0;
                            $this->setGenieTransGoogle($transMsg);
                            $errorMsg = __('You have not selected Merchant yet.', 'geniewords');
                            $this->setTransientErrorMsg($errorMsg);
                        }
                    }
                }
            }
        }

        function get_current_user_info() {
            $get_user_id = get_post_meta(1, 'geniewords_userid');

            if (!defined('GENIEWORDS_ID_TOKEN')) {
                $id_token = Geniewords_js::get_idToken();
            } else {
                $id_token = GENIEWORDS_ID_TOKEN;
            }

            $args_get = array(
                'method' => 'GET',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    'Authorization' => 'Bearer ' . $id_token
                )
            );

            $basicinfo_url = self::$_genieWordApiUrl . '/users/' . $get_user_id[0] . '/basicInfo';
            $response_get = wp_remote_get($basicinfo_url, $args_get);

            $httpResponse = wp_remote_retrieve_response_code($response_get);

            if ($httpResponse !== 200) {
                $errorMsg = __('Error occurred in getting user basic info.', 'geniewords');
                $this->setTransientErrorMsg($errorMsg);
                return null;
            } else {
                $currentUserInfo = json_decode($response_get['body']);
                return $currentUserInfo;
            }
        }

        function put_google_shopping_metatag_to_head()
        {
            $currentUserInfo = $this->get_current_user_info();

            if ($currentUserInfo && $currentUserInfo->googleShoppingMetaTag) {
                $googleShoppingMetaTag = str_replace("\\", '', $currentUserInfo->googleShoppingMetaTag);

                update_post_meta(1, 'geniewords_google_shopping_meta_tag', $googleShoppingMetaTag);

                echo $googleShoppingMetaTag;
            }
        }

        function put_facebook_metatag_to_head()
        {
            $get_user_id = get_post_meta(1, 'geniewords_userid');

            if (!defined('GENIEWORDS_ID_TOKEN')) {
                $id_token = Geniewords_js::get_idToken();
            } else {
                $id_token = GENIEWORDS_ID_TOKEN;
            }

            $args_get = array(
                'method' => 'GET',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    'Authorization' => 'Bearer ' . $id_token
                )
            );

            $basicinfo_url = self::$_genieWordApiUrl . '/users/' . $get_user_id[0] . '/basicInfo';
            $response_get = wp_remote_get($basicinfo_url, $args_get);

            $httpResponse = wp_remote_retrieve_response_code($response_get);

            if ($httpResponse !== 200) {
                $errorMsg = __('Error occurred in getting user basic info.', 'geniewords');
                $this->setTransientErrorMsg($errorMsg);
            } else {
                $get_refresh_token = json_decode($response_get['body']);

                if ($get_refresh_token && $get_refresh_token->facebookData && $get_refresh_token->facebookData->facebookMetaTag) {
                    $facebookMetaTag = str_replace("\\", '', $get_refresh_token->facebookData->facebookMetaTag);
                    echo $facebookMetaTag;
                }
            }
        }

        function update_add_analytics_to_head() {
            $get_geniewords_add_analytics_in_head = $_POST['addGoogleAnalyticsInHead'];

            update_post_meta(1, 'add_analytics_in_head', $get_geniewords_add_analytics_in_head);
        }

        function update_analytics_custom_value() {
            $get_geniewords_analytics_custom_value = $_POST['customAnalyticsValue'];

            update_post_meta(1, 'analytics_custom_value', $get_geniewords_analytics_custom_value);
        }

        function update_pixel_custom_value() {
            $get_geniewords_pixel_custom_value = $_POST['customPixelValue'];

            update_post_meta(1, 'pixel_custom_value', $get_geniewords_pixel_custom_value);
        }

        function update_add_facebook_pixel_to_head() {
            $get_geniewords_add_facebook_pixel_in_head = $_POST['addFacebookPixelInHead'];

            update_post_meta(1, 'add_facebook_pixel_in_head', $get_geniewords_add_facebook_pixel_in_head);
        }

        function add_fb_pixel_code_in_head()
        {
            $get_add_facebook_pixel_in_head = get_post_meta(1, 'add_facebook_pixel_in_head');

            if (empty($get_add_facebook_pixel_in_head) || $get_add_facebook_pixel_in_head[0] == 'NONE') {
                echo '';
                return;
            }

            $get_pixel_custom_value = get_post_meta(1, 'pixel_custom_value');

            if ($get_add_facebook_pixel_in_head[0] == 'CUSTOM') {
                echo $get_pixel_custom_value[0];
                return;
            }

            $get_user_id = get_post_meta(1, 'geniewords_userid');

            if (!defined('GENIEWORDS_ID_TOKEN')) {
                $id_token = Geniewords_js::get_idToken();
            } else {
                $id_token = GENIEWORDS_ID_TOKEN;
            }

            $args_get = array(
                'method' => 'GET',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    'Authorization' => 'Bearer ' . $id_token
                )
            );

            $basicinfo_url = self::$_genieWordApiUrl . '/users/' . $get_user_id[0] . '/basicInfo';
            $response_get = wp_remote_get($basicinfo_url, $args_get);

            $httpResponse = wp_remote_retrieve_response_code($response_get);

            if ($httpResponse !== 200) {
                $errorMsg = __('Error occurred in getting user basic info.', 'geniewords');
                $this->setTransientErrorMsg($errorMsg);
            } else {
                $get_refresh_token = json_decode($response_get['body']);

                if ($get_refresh_token
                    && $get_refresh_token->facebookAuthInfo
                    && $get_refresh_token->facebookAuthInfo->pixelInfo
                    && $get_refresh_token->facebookAuthInfo->pixelInfo->code) {

                    echo $get_refresh_token->facebookAuthInfo->pixelInfo->code;
                }
            }
        }

        function add_analytics_in_head()
        {
            $get_add_analytics_in_head = get_post_meta(1, 'add_analytics_in_head');

            if (empty($get_add_analytics_in_head) || $get_add_analytics_in_head[0] == 'NONE') {
                echo '';
                return;
            }

            $get_analytics_custom_value = get_post_meta(1, 'analytics_custom_value');

            if ($get_add_analytics_in_head[0] == 'CUSTOM') {
                echo $get_analytics_custom_value[0];
                return;
            }

            $get_user_id = get_post_meta(1, 'geniewords_userid');

            if (!defined('GENIEWORDS_ID_TOKEN')) {
                $id_token = Geniewords_js::get_idToken();
            } else {
                $id_token = GENIEWORDS_ID_TOKEN;
            }

            $args_get = array(
                'method' => 'GET',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    'Authorization' => 'Bearer ' . $id_token
                )
            );

            $basicinfo_url = self::$_genieWordApiUrl . '/users/' . $get_user_id[0] . '/basicInfo';

            $response_get = wp_remote_get($basicinfo_url, $args_get);

            $httpResponse = wp_remote_retrieve_response_code($response_get);

            if ($httpResponse !== 200) {
                $errorMsg = __('Error occurred in getting user basic info.', 'geniewords');
                $this->setTransientErrorMsg($errorMsg);
            } else {
                $get_refresh_token = json_decode($response_get['body']);

                if ($get_refresh_token
                    && $get_refresh_token->isAnalyticsAttached
                    && $get_refresh_token->analyticsViewId
                    && $get_refresh_token->analyticsId) {

                    echo '<!-- Global site tag (gtag.js) - Google Analytics -->
                    <script async src="https://www.googletagmanager.com/gtag/js?id=' . $get_refresh_token->analyticsId . '"></script>
                    <script>
                      window.dataLayer = window.dataLayer || [];
                      function gtag(){dataLayer.push(arguments);}
                      gtag("js", new Date());
                      gtag("config", "' . $get_refresh_token->analyticsId . '");
                    </script>';
                }
            }
        }

        function add_woocommerce_fb_products()
        {
            if (!defined('GENIEWORDS_ID_TOKEN')) {
                $id_token = Geniewords_js::get_idToken();
            } else {
                $id_token = GENIEWORDS_ID_TOKEN;
            }

            if ($id_token) {
                $get_user_id = get_post_meta(1, 'geniewords_userid');
                $get_fb_longlive_token = get_post_meta(1, 'geniewords_fb_longlive_token');
                $args_get = array(
                    'method' => 'GET',
                    'timeout' => 120,
                    'httpversion' => '1.1',
                    'headers' => array(
                        'Authorization' => 'Bearer ' . $id_token
                    )
                );

                $catalouge_url = self::$_genieWordApiUrl . '/facebook/' . $get_user_id[0] . '/productCatalogs';
                $get_catalouge = wp_remote_get($catalouge_url, $args_get);

                $httpResponse = wp_remote_retrieve_response_code($get_catalouge);

                if ($httpResponse == 200) {
                    $transMsg['facebookProductCatalog'] = 1;
                    $this->setGenieTransFacebook($transMsg);

                    //Fetch Basic Info
                    $basicinfo_url = self::$_genieWordApiUrl . '/users/' . $get_user_id[0] . '/basicInfo';
                    $response_get = wp_remote_get($basicinfo_url, $args_get);

                    $httpResponse = wp_remote_retrieve_response_code($response_get);

                    if ($httpResponse !== 200) {
                        $errorMsg = __('Error occurred in getting user basic info.', 'geniewords');
                        $this->setTransientErrorMsg($errorMsg);
                    } else {
                        $get_refresh_token = json_decode($response_get['body']);
                        $company_name = $get_refresh_token->company . ' Product Catalog';

                        $facebookMetaTag = $_POST['facebookMetaTag'];
                        if($facebookMetaTag && isset($facebookMetaTag)) {
                            $facebookData = $get_refresh_token->facebookData;
                            $facebookData->facebookMetaTag = '' . $_POST['facebookMetaTag'];

                            $data_to_be_updated = array('facebookData' => $facebookData);

                            $args_website = array(
                                'method' => 'PUT',
                                'timeout' => 120,
                                'httpversion' => '1.1',
                                'headers' => array(
                                    'Content-Type' => 'application/json; charset=utf-8',
                                    'Authorization' => 'Bearer ' . $id_token
                                ),
                                'body' => json_encode($data_to_be_updated)
                            );

                            $updateUserUrl = self::$_genieWordApiUrl . '/users/' . $get_user_id[0];
                            $response_post = wp_remote_post($updateUserUrl, $args_website);
                            $httpResponse = wp_remote_retrieve_response_code($response_post);

                            if ($httpResponse !== 200) {
                                echo "<pre>";
                                print_r('Error while updating meta tag');
                            }
                        }
                    }

                    if (isset($get_refresh_token) && $get_refresh_token->facebookAuthInfo
                        && (!$get_refresh_token->facebookAuthInfo->productCatalog
                        || !$get_refresh_token->facebookAuthInfo->productCatalog->id
                        || !isset($get_refresh_token->facebookAuthInfo->productCatalog->id))) {
                        if (isset($company_name)) {
                            $body_data = array(
                                'name' => $company_name
                            );
                        } else {
                            $body_data = array(
                                'name' => GENIEWORDS_FB_NAME . ' Product Catalog'
                            );
                        }

                        $args_website = array(
                            'method' => 'POST',
                            'timeout' => 120,
                            'httpversion' => '1.1',
                            'headers' => array('Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $id_token),
                            'body' => json_encode($body_data)
                        );

                        $response_post = wp_remote_post(self::$_genieWordApiUrl . "/facebook/" . $get_user_id[0] . "/product-catalog/assignUser", $args_website);
                        $httpResponse = wp_remote_retrieve_response_code($response_post);

                        if ($httpResponse == 200) {
                            $transMsg['facebookProductCatalogAdded'] = 1;
                            $this->setGenieTransFacebook($transMsg);
                            $get_val_response = json_decode($response_post['body']);

                            if ($get_val_response->id && count($get_fb_longlive_token) > 0) {
                                $this->create_product_set();
                                $this->create_facebook_datafeed($get_val_response->id, $get_fb_longlive_token[0]);
                                $this->create_datafeed_xml_file($get_refresh_token->company, $get_refresh_token->website, 'FACEBOOK');
                            }
                        } else {
                            $transMsg['facebookProductCatalogAdded'] = 0;
                            $this->setGenieTransFacebook($transMsg);
                            $errorMsg = __('Error occurred at adding Facebook Product Catalog.', 'geniewords');
                            $this->setTransientErrorMsg($errorMsg);
                        }
                    }
                } else {
                    $transMsg['facebookProductCatalog'] = 0;
                    $this->setGenieTransFacebook($transMsg);
                    $errorMsg = __('Error occurred at getting facebook product catalog.', 'geniewords');
                    $this->setTransientErrorMsg($errorMsg);
                }
            }
        }

        function create_product_set() {
            $get_user_id = get_post_meta(1, 'geniewords_userid');

            if (!defined('GENIEWORDS_ID_TOKEN')) {
                $id_token = Geniewords_js::get_idToken();
            } else {
                $id_token = GENIEWORDS_ID_TOKEN;
            }

            $args_create_product_set = array(
                'method' => 'POST',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    "Authorization" => "Bearer " . $id_token,
                    "Content-Type" => "application/json"
                ),
                'body' => json_encode(array("name" => 'All Products'))
            );

            $create_productset_url = self::$_genieWordApiUrl . '/facebook/' . $get_user_id[0] . '/productSets';
            $response_get = wp_remote_get($create_productset_url, $args_create_product_set);

            $httpResponse = wp_remote_retrieve_response_code($response_get);

            if ($httpResponse !== 200) {
                $errorMsg = __('Error occurred in creating product set.', 'geniewords');
                $this->setTransientErrorMsg($errorMsg);
            }
        }

        function delete_product_hook_function($product_id) {
            $this->update_product_hook_function($product_id, 'DELETE');
        }

        function add_product_hook_function($product_id) {
            $this->update_product_hook_function($product_id, 'ADD');
        }

        function update_product_hook_function($product_id, $action = 'UPDATE')
        {
            //Fetch Basic Info
            $get_user_id = get_post_meta(1, 'geniewords_userid');

            if (!defined('GENIEWORDS_ID_TOKEN')) {
                $id_token = Geniewords_js::get_idToken();
            } else {
                $id_token = GENIEWORDS_ID_TOKEN;
            }

            $args_get = array(
                'method' => 'GET',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    'Authorization' => 'Bearer ' . $id_token
                )
            );

            $basicinfo_url = self::$_genieWordApiUrl . '/users/' . $get_user_id[0] . '/basicInfo';
            $response_get = wp_remote_get($basicinfo_url, $args_get);

            $httpResponse = wp_remote_retrieve_response_code($response_get);

            if ($httpResponse !== 200) {
                $errorMsg = __('Error occurred in getting user basic info.', 'geniewords');
                $this->setTransientErrorMsg($errorMsg);
            } else {
                $get_refresh_token = json_decode($response_get['body']);
                $this->create_datafeed_xml_file($get_refresh_token->company, $get_refresh_token->website, '', $action === 'DELETE' ? $product_id : '');
            }
        }

        function create_datafeed($merchantId)
        {
            // RETRIEVE EXISTING DATAFEEDS ON CUSTOMER'S MERCHANT CENTER
            $list_datafeed_url = 'https://shoppingcontent.googleapis.com/content/v2.1/' . $merchantId . '/datafeeds';
            $args_get = array(
                'method' => 'GET',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    'Authorization' => 'Bearer ' . GENIE_ACCESS_TOKEN
                )
            );

            $response_get = wp_remote_get($list_datafeed_url, $args_get);

            $httpResponse = wp_remote_retrieve_response_code($response_get);

            if ($httpResponse !== 200) {
                $errorMsg = __('Error occurred in getting list of datafeeds.', "geniewords");
                $this->setTransientErrorMsg($errorMsg);
                return false;
            } else {
                $datafeeds = json_decode($response_get['body']);

                $dataFeedExists = false;
                if($datafeeds && $datafeeds->resources) {
                    foreach($datafeeds->resources as $datafeed) {
                        if ($datafeed->name === 'Geniewords products') {
                            $dataFeedExists = true;
                            break;
                        }
                    }
                }

                // CREATE DATA FEED IF DOESN'T EXISTS
                if (!$dataFeedExists) {
                    // FIRST TRY with SurfacesAcrossGoogle and Shopping destinations
                    $httpResponse = $this->create_data_feed_in_google([
                        "SurfacesAcrossGoogle",
                        "Shopping"
                    ], $merchantId);

                    if ($httpResponse !== 200) {
                        // SECOND TRY without Shopping destination
                        $httpResponse = $this->create_data_feed_in_google([
                            "SurfacesAcrossGoogle"
                        ], $merchantId);

                        if ($httpResponse !== 200) {
                            $transMsg['createDataFeed'] = 0;
                            $this->setGenieTransGoogle($transMsg);
                            $errorMsg = __('Error while creating datafeed.', "geniewords");
                            $this->setTransientErrorMsg($errorMsg);
                        } else {
                            $this->process_after_data_feed_created();
                        }
                    } else {
                        $this->process_after_data_feed_created();
                    }
                } else {
                    $datafeed_id = $datafeeds && $datafeeds->resources[0] && $datafeeds->resources[0]->id ? $datafeeds->resources[0]->id : null;
                    $this->update_datafeed($merchantId, $datafeed_id);
                }

                return $dataFeedExists;
            }
        }

        function process_after_data_feed_created() {
            $transMsg['createDataFeed'] = 1;
            $this->setGenieTransGoogle($transMsg);

            $get_user_id = get_post_meta(1, 'geniewords_userid');

            $admin_activity_log = array(
                'label' => 'CREATE_DATA_FEED',
                'adminId' => 'PLUGIN',
                'adminName' => 'PLUGIN',
                'actionText' => 'Datafeed is created.',
                'customerId' => $get_user_id[0]
            );

            if (!defined('GENIEWORDS_ID_TOKEN')) {
                $id_token = Geniewords_js::get_idToken();
            } else {
                $id_token = GENIEWORDS_ID_TOKEN;
            }

            $args_create_admin_activity = array(
                'method' => 'POST',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    "Authorization" => "Bearer " . $id_token,
                    "Content-Type" => "application/json"
                ),
                'body' => json_encode($admin_activity_log)
            );


            $create_admin_activity_url = self::$_genieWordApiUrl . '/activities/' . $get_user_id[0];

            $response_create_admin_activity = wp_remote_post($create_admin_activity_url, $args_create_admin_activity);

            $httpResponse = wp_remote_retrieve_response_code($response_create_admin_activity);

            if ($httpResponse !== 200) {
                $transMsg['createDataFeed'] = 0;
                $this->setGenieTransGoogle($transMsg);
                $errorMsg = json_decode($response_create_admin_activity['body'], true)['message'];
                $this->setTransientErrorMsg($errorMsg);
            } else {
                echo __('Your Google Shopping is now configured correct.', "geniewords");
            }
        }

        function create_data_feed_in_google($includedDestinations, $merchantId) {
            $get_country = explode("_", $_POST['country']);
            $paused = false;

            $randomUUID = wp_generate_uuid4();
            $fileName = 'geniewords-products-' . $randomUUID;

            $datafeed_info = array(
                "name" => "Geniewords products",
                "contentType" => "products",
                "kind" => "content#datafeed",
                "fileName" => $fileName,
                "fetchSchedule" => array(
                    "paused" => $paused,
                    "hour" => 13,
                    "timeZone" => "Europe/Copenhagen",
                    "fetchUrl" => self::$_websiteUrl . "wp-content/uploads/geniewords-products.xml"
                ),
                "targets" => array([
                    "country" => $get_country[1],
                    "language" => $get_country[0],
                    "includedDestinations" => $includedDestinations
                ])
            );

            $args_create_datafeed = array(
                'method' => 'POST',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    "Authorization" => "Bearer " . GENIE_ACCESS_TOKEN,
                    "Content-Type" => "application/json"
                ),
                'body' => json_encode($datafeed_info)
            );

            $createDataFeedUrl = 'https://shoppingcontent.googleapis.com/content/v2.1/' . $merchantId . '/datafeeds';

            $response_post = wp_remote_post($createDataFeedUrl, $args_create_datafeed);

            $httpResponse = wp_remote_retrieve_response_code($response_post);

            return $httpResponse;
        }

        function update_datafeed($merchantId, $feedId)
        {
            // RETRIEVE EXISTING DATAFEEDS ON CUSTOMER'S MERCHANT CENTER
            $list_datafeed_url = 'https://shoppingcontent.googleapis.com/content/v2.1/' . $merchantId . '/datafeeds/' . $feedId;
            $args_get = array(
                'method' => 'GET',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    'Authorization' => 'Bearer ' . GENIE_ACCESS_TOKEN
                )
            );

            $response_get = wp_remote_get($list_datafeed_url, $args_get);

            $httpResponse = wp_remote_retrieve_response_code($response_get);

            if ($httpResponse !== 200) {
                $errorMsg = __('Error occurred in getting list of datafeeds.', "geniewords");
                $this->setTransientErrorMsg($errorMsg);
                return false;
            } else {
                $existing_datafeed_info = json_decode($response_get['body']);

                $get_country = explode("_", $_POST['country']);

                $paused = false;

                $datafeed_info = array(
                    "id" => $feedId,
                    "name" => "Geniewords products",
                    "contentType" => "products",
                    "kind" => "content#datafeed",
                    "fileName" => $existing_datafeed_info->fileName,
                    "fetchSchedule" => array(
                        "paused" => $paused,
                        "hour" => 13,
                        "timeZone" => "Europe/Copenhagen",
                        "fetchUrl" => self::$_websiteUrl . "wp-content/uploads/geniewords-products.xml"
                    ),
                    "targets" => array([
                        "country" => $get_country[1],
                        "language" => $get_country[0],
                        "includedDestinations" => [
                            "SurfacesAcrossGoogle",
                            "Shopping"
                        ]
                    ])
                );

                $args_update_datafeed = array(
                    'method' => 'PUT',
                    'timeout' => 120,
                    'httpversion' => '1.1',
                    'headers' => array(
                        "Authorization" => "Bearer " . GENIE_ACCESS_TOKEN,
                        "Content-Type" => "application/json"
                    ),
                    'body' => json_encode($datafeed_info)
                );

                $update_datafeed_url = 'https://shoppingcontent.googleapis.com/content/v2.1/' . $merchantId . '/datafeeds/' . $feedId;

                $response_put = wp_remote_post($update_datafeed_url, $args_update_datafeed);

                $httpResponse = wp_remote_retrieve_response_code($response_put);

                if ($httpResponse !== 200) {
                    $transMsg['createDataFeed'] = 0;
                    $this->setGenieTransGoogle($transMsg);
                    $errorMsg = json_decode($response_put['body'], true)['message'];
                    $this->setTransientErrorMsg($errorMsg);
                } else {
                    $transMsg['createDataFeed'] = 1;
                    $this->setGenieTransGoogle($transMsg);
                }
            }
        }

        function create_facebook_datafeed($productCatalogId, $long_lived_token)
        {
            // RETRIEVE EXISTING DATAFEEDS ON CUSTOMER'S MERCHANT CENTER
            $list_datafeed_url = 'https://graph.facebook.com/' . $productCatalogId . '/product_feeds?access_token=' . $long_lived_token;
            $args_get = array(
                'method' => 'GET',
                'timeout' => 120,
                'httpversion' => '1.1'
            );

            $response_get = wp_remote_get($list_datafeed_url, $args_get);

            $httpResponse = wp_remote_retrieve_response_code($response_get);

            if ($httpResponse !== 200) {
                $errorMsg = __('Error occurred in getting list of datafeeds for product catalog.', "geniewords");
                $this->setTransientErrorMsg($errorMsg);
                return false;
            } else {
                $datafeeds = json_decode($response_get['body']);

                $dataFeedExists = false;
                if($datafeeds && $datafeeds->data) {
                    foreach($datafeeds->data as $datafeed) {
                        if ($datafeed->name === 'Geniewords products') {
                            $dataFeedExists = true;
                            break;
                        }
                    }
                }

                // CREATE DATA FEED IF DOESN'T EXISTS
                if (!$dataFeedExists) {
                    $get_country = explode("_", $_POST['country']);
                    $datafeed_info = array(
                        "name" => "Geniewords products",
                        "schedule" => array(
                            "interval" => "HOURLY",
                            "minute" => 5,
                            "timezone" => "Europe/Copenhagen",
                            "url" => self::$_websiteUrl . "wp-content/uploads/geniewords-products.xml"
                        )
                    );

                    $args_create_datafeed = array(
                        'method' => 'POST',
                        'timeout' => 120,
                        'httpversion' => '1.1',
                        'headers' => array(
                            "Content-Type" => "application/json"
                        ),
                        'body' => json_encode($datafeed_info)
                    );

                    $createDataFeedUrl = 'https://graph.facebook.com/' . $productCatalogId . '/product_feeds?default_currency=DKK&access_token=' . $long_lived_token;

                    $response_post = wp_remote_post($createDataFeedUrl, $args_create_datafeed);

                    $httpResponse = wp_remote_retrieve_response_code($response_post);

                    if ($httpResponse !== 200) {
                        $transMsg['createDataFeed'] = 0;
                        $this->setGenieTransGoogle($transMsg);
                        $errorMsg = json_decode($response_post['body'], true)['message'];
                        $this->setTransientErrorMsg($errorMsg);
                    } else {
                        $transMsg['createDataFeed'] = 1;
                        $this->setGenieTransGoogle($transMsg);
                    }
                }

                return $dataFeedExists;
            }
        }

        function create_datafeed_xml_file($company, $website, $product_type, $product_id = '') {
            $products = $this->prepareProductsCollectionArr($product_id);

            $xml = new SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><rss xmlns:g="http://base.google.com/ns/1.0" version="2.0"/>');

            $channel = $xml->addChild('channel');
            $title = $channel->addChild('title', $company);
            $link = $channel->addChild('link', $website);
            $description = $channel->addChild('description', 'GenieWords WooCommerce plugin - This product feed is created with GenieWords WooCommerce plugin from https://genieWords.com For all your support questions contact us on https://genieWords.com');

            foreach ($products[0] as $product) {
                $productInfo = $product[0];
                $get_country = explode("_", $_POST['country']);
                $product_description = preg_replace('/&(?!#?[a-z0-9]+;)/', '&amp;', strip_tags(html_entity_decode($productInfo['description'])));

                $item = $channel->addChild('item');
                $id = $item->addChild('xmlns:g:id', $productInfo['offerId']);
                $title = $item->addChild('xmlns:g:title', $productInfo['title']);
                $description = $item->addChild('xmlns:g:description', $product_description);
                $link = $item->addChild('xmlns:g:link', $productInfo['link']);
                $image_link = $item->addChild('xmlns:g:image_link', $productInfo['imageLink']);
                $availability = $item->addChild('xmlns:g:availability', $productInfo['availability']);
                $price = $item->addChild('xmlns:g:price', $productInfo['price']['currency'] .' '. $productInfo['price']['value']);
                $condition = $item->addChild('xmlns:g:condition', $productInfo['condition']);
                $identifier_exists = $item->addChild('xmlns:g:identifier_exists', 'no');

                if($productInfo['salePrice']) {
                    $sale_price = $item->addChild('xmlns:g:sale_price', $productInfo['salePrice']['currency'] .' '. $productInfo['salePrice']['value']);
                }

                $images = $productInfo['images'];

                foreach ($images as $image) {
                    $item->addChild('xmlns:g:additional_image_link', $image);
                }
            }

            $ourFileName = "geniewords-products.xml";
            unlink("../wp-content/uploads/".$ourFileName);

            $myfile = fopen("../wp-content/uploads/" .$ourFileName, 'w');
            fwrite($myfile, $xml->asXML());
            fclose($myfile);

            // UPDATES POST META
            $get_google_meta = get_post_meta(1, 'geniewords_google_shopping_status');
            if (empty($get_google_meta[0])) {
                update_post_meta(1, 'geniewords_google_shopping_status', 1);
            } else {
                update_post_meta(1, 'geniewords_google_shopping_status', 1);
            }

            if (!empty($_POST['country'])) {
                $get_geniewords_country_name = get_post_meta(1, 'geniewords_country_name');

                if (empty($get_geniewords_country_name[0])) {
                    add_post_meta(1, 'geniewords_country_name', $_POST['country']);
                } else {
                    update_post_meta(1, 'geniewords_country_name', $_POST['country']);
                }
            }

            $description_option = $_POST['descriptionoption'];

            if (!isset($description_option)) {
                $description_option = get_post_meta(1, 'get_geniewords_description_type')[0];
            }

            if ($description_option == 1) {
                update_post_meta(1, 'get_geniewords_description_type', 1);
            } else {
                update_post_meta(1, 'get_geniewords_description_type', 2);
            }

            if ($product_type === 'GOOGLE_SHOPPING') {
                $get_geniewords_variant_status = $_POST['variants'];

                if ($get_geniewords_variant_status == '') {
                    update_post_meta(1, 'geniewords_variant_status', false);
                } else {
                    update_post_meta(1, 'geniewords_variant_status', true);
                }
            } else if ($product_type === 'FACEBOOK') {
                $get_geniewords_fb_variant_status = $_POST['fb_variants'];

                if ($get_geniewords_fb_variant_status == '') {
                    update_post_meta(1, 'geniewords_fb_variant_status', false);
                } else {
                    update_post_meta(1, 'geniewords_fb_variant_status', true);
                }
            }

            if (isset($_POST['name']) && !empty($_POST['name'])) {
                $get_ajax_manually = get_post_meta(1, 'geniewords_ajax_manually_status');
                if (empty($get_ajax_manually[0])) {
                    update_post_meta(1, 'geniewords_ajax_manually_status', 1);
                } else {
                    update_post_meta(1, 'geniewords_ajax_manually_status', 1);
                }
                echo "<b style='color:green; line-height: 35px;'>" . __('Changes has been saved successfully.', 'geniewords') . " </b><br/>";
                exit;
            }
        }

        // Delete Transient
        function deactive_javascript_code()
        {
            $this->deletePluginInfoFromDb();
        }

        // Add Claim Website Meta Tag //
        function add_meta_tags()
        {
            $geniewords_user_web_meta = get_post_meta(1, 'geniewords_user_web_meta');
            if (isset($geniewords_user_web_meta)) {
                echo htmlspecialchars_decode($geniewords_user_web_meta[0]);
            }
        }

        function get_idToken($init = 0, $userEmail = '', $userPassword = '')
        {
            if ($init == 0) {
                $user_email = get_post_meta(1, 'geniewords_user_email');
                $user_email = current($user_email);
                $user_password = get_post_meta(1, 'geniewords_user_password');
                $user_password = current($user_password);
            } else {
                $user_email = $userEmail;
                $user_password = $userPassword;
            }

            $loginDetails = array('email' => $user_email, 'password' => $user_password, 'returnSecureToken' => true);
            $args_website = array(
                'method' => 'POST',
                'timeout' => 120,
                'httpversion' => '1.1',
                'headers' => array(
                    'Content-Type' => 'application/json; charset=utf-8',
                ),
                'body' => json_encode($loginDetails)
            );
            $response_post = wp_remote_post(self::$_genieWordApiUrl . '/login', $args_website);
            $httpResponse = wp_remote_retrieve_response_code($response_post);

            if ($httpResponse == 200) {
                $transMsg['tokenGenerated'] = 1;
                $this->setGenieTrans($transMsg);

                $get_response_data = json_decode($response_post['body']);
                $getToken = isset($get_response_data->idToken) ? $get_response_data->idToken : '';
                if ($getToken !== '') {
                    if ($init == 1) {
                        return $get_response_data;
                    }
                    define('GENIEWORDS_ID_TOKEN', $getToken);
                    return $getToken;
                }
            } else {
                $transMsg['tokenGenerated'] = 0;
                $this->setGenieTrans($transMsg);
                set_transient('api_login_error', 'Error');
                return false;
            }
        }

        function signout_users()
        {
            $this->deletePluginInfoFromDb();
            echo "success"; exit;
        }

        public static function init()
        {

            static $instance = null;

            if (!$instance) {
                $instance = new Geniewords_js();
            }

            return $instance;

        }

        /**
         * @param $user_id
         * @param $refresh_token
         * @param $merchantId
         * @param $company_name
         * @param $shopping_email
         * @param $fb_short_token
         * @param $fb_longlive_token
         * @param $fb_business_id
         * @param $fb_pixelinfo_code
         * @param $fb_user_business_id
         */
        protected function setPostMetaFieldValuesForFb($user_id, $company_name, $fb_short_token, $fb_longlive_token, $fb_business_id, $fb_pixelinfo_code, $fb_user_business_id)
        {
            $get_user_id = get_post_meta(1, 'geniewords_userid');
            $get_geniewords_user_company = get_post_meta(1, 'geniewords_user_company');
            $get_fb_short_token = get_post_meta(1, 'geniewords_fb_short_token');
            $get_fb_longlive_token = get_post_meta(1, 'geniewords_fb_longlive_token');
            $get_fb_business_id = get_post_meta(1, 'geniewords_fb_business_id');
            $get_fb_pixelinfo_code = get_post_meta(1, 'geniewords_fb_pixelinfo_code');
            $get_fb_user_business_id = get_post_meta(1, 'geniewords_fb_user_business_id');

            if (empty($get_user_id[0]) && !empty($user_id)) {
                add_post_meta(1, 'geniewords_userid', $user_id);
            } else if (!empty($user_id)) {
                update_post_meta(1, 'geniewords_userid', $user_id);
            }

            if (empty($get_geniewords_user_company[0]) && !empty($company_name)) {
                add_post_meta(1, 'geniewords_user_company', $company_name);
            } else if (!empty($company_name)) {
                update_post_meta(1, 'geniewords_user_company', $company_name);
            }

            if (empty($get_fb_short_token[0]) && !empty($fb_short_token)) {
                add_post_meta(1, 'geniewords_fb_short_token', $fb_short_token);
            } else if (!empty($fb_short_token)) {
                update_post_meta(1, 'geniewords_fb_short_token', $fb_short_token);
            }

            if (empty($get_fb_longlive_token[0]) && !empty($fb_longlive_token)) {
                add_post_meta(1, 'geniewords_fb_longlive_token', $fb_longlive_token);
            } else if (!empty($fb_longlive_token)) {
                update_post_meta(1, 'geniewords_fb_longlive_token', $fb_longlive_token);
            }

            if (empty($get_fb_business_id[0]) && !empty($fb_business_id)) {
                add_post_meta(1, 'geniewords_fb_business_id', $fb_business_id);
            } else if (!empty($fb_business_id)) {
                update_post_meta(1, 'geniewords_fb_business_id', $fb_business_id);
            }

            if (empty($get_fb_pixelinfo_code[0]) && !empty($fb_pixelinfo_code)) {
                add_post_meta(1, 'geniewords_fb_pixelinfo_code', $fb_pixelinfo_code);
            } else if (!empty($fb_pixelinfo_code)) {
                update_post_meta(1, 'geniewords_fb_pixelinfo_code', $fb_pixelinfo_code);
            }

            if (empty($get_fb_user_business_id[0]) && !empty($fb_user_business_id)) {
                add_post_meta(1, 'geniewords_fb_user_business_id', $fb_user_business_id);
            } else if (!empty($fb_user_business_id)) {
                update_post_meta(1, 'geniewords_fb_user_business_id', $fb_user_business_id);
            }
        }

        protected function setPostMetaFieldValuesForGoogle($user_id, $refresh_token, $merchantId, $company_name, $shopping_email)
        {
            $get_user_id = get_post_meta(1, 'geniewords_userid');
            $get_refresh_token_id = get_post_meta(1, 'geniewords_refresh_token');
            $get_merchantId = get_post_meta(1, 'geniewords_merchantid');
            $get_geniewords_user_company = get_post_meta(1, 'geniewords_user_company');
            $get_geniewords_shopping_email = get_post_meta(1, 'geniewords_shopping_email');

            if (empty($get_user_id[0]) && !empty($user_id)) {
                add_post_meta(1, 'geniewords_userid', $user_id);
            } else if (!empty($user_id)) {
                update_post_meta(1, 'geniewords_userid', $user_id);
            }

            if (empty($get_refresh_token_id[0]) && !empty($refresh_token)) {
                add_post_meta(1, 'geniewords_refresh_token', $refresh_token);
            } else if (!empty($refresh_token)) {
                update_post_meta(1, 'geniewords_refresh_token', $refresh_token);
            }

            if (empty($get_merchantId[0]) && !empty($merchantId)) {
                add_post_meta(1, 'geniewords_merchantid', $merchantId);
            } else if (!empty($merchantId)) {
                update_post_meta(1, 'geniewords_merchantid', $merchantId);
            }

            if (empty($get_geniewords_user_company[0]) && !empty($company_name)) {
                add_post_meta(1, 'geniewords_user_company', $company_name);
            } else if (!empty($company_name)) {
                update_post_meta(1, 'geniewords_user_company', $company_name);
            }

            if (empty($get_geniewords_shopping_email[0]) && !empty($shopping_email)) {
                add_post_meta(1, 'geniewords_shopping_email', $shopping_email);
            } else if (!empty($shopping_email)) {
                update_post_meta(1, 'geniewords_shopping_email', $shopping_email);
            }
        }

        /**
         * @return int
         */
        function enableDisableVariantsCheckbox()
        {
            $args = array(
                'post_type'      => 'product',
                'posts_per_page' => 10,
                'product_cat'    => 'hoodies'
            );

            $get_products = new WP_Query( $args );

            while ($get_products->have_posts()) :
                $get_products->the_post();

                $get_geniewords_variant_status = get_post_meta(1, 'geniewords_variant_status');
                $get_geniewords_fb_variant_status = get_post_meta(1, 'geniewords_fb_variant_status');

                $variations = get_posts($args);
                if (empty($get_geniewords_variant_status)) {
                    if (count($variations) > 0) {
                        update_post_meta(1, 'geniewords_variant_status', true);
                    } else {
                        update_post_meta(1, 'geniewords_variant_status', false);
                    }
                }

                if (empty($get_geniewords_fb_variant_status)) {
                    if (count($variations) > 0) {
                        update_post_meta(1, 'geniewords_fb_variant_status', true);
                    } else {
                        update_post_meta(1, 'geniewords_fb_variant_status', false);
                    }
                }
            endwhile;
        }

        /**
         * @param $product_id
         * @return array
         */
        protected function prepareProductsCollectionArr($product_id = '')
        {
            $products = array();
            $args = array(
                'post_type' => 'product',
                'posts_per_page' => -1,
                'post_status' => 'publish'
            );

            $get_products = new WP_Query($args);
            while ($get_products->have_posts()) :
                $get_products->the_post();

                $id = get_the_id();

                if($id === $product_id) continue;

                $wc_product = new WC_Product($id);

                $description_option = $_POST['descriptionoption'];

                if (!isset($description_option)) {
                    $description_option = get_post_meta(1, 'get_geniewords_description_type')[0];
                }

                $shortDescription = $wc_product->get_short_description();
                $longDescription = $wc_product->get_description();

                if(isset($description_option) && (!empty($shortDescription) || !empty($longDescription))) {
                    if ($description_option == '1') {
                        $description = $shortDescription ? $shortDescription : $longDescription;
                    } else if ($description_option == '2') {
                        $description = $longDescription ? $longDescription : $shortDescription;
                    }
                }

                $gtin = $wc_product->get_sku();
                $sku = $wc_product->get_sku();

                $brand = get_the_terms($id, 'product_cat');
                $brand = esc_attr($brand[0]->name);

                $woo_currency = get_woocommerce_currency();
                $get_country = explode("_", $_POST['country']);
                $link = get_permalink();
                $availability = $wc_product->get_stock_status();
                if ($availability == 'instock') {
                    $availability = "in stock";
                } else {
                    $availability = "out of stock";
                }

                $image_link = wp_get_attachment_url(get_post_thumbnail_id($id));
                $condition = "new";

                $variant_option = $_POST['variants'];

                if (!isset($variant_option)) {
                    $variant_option = get_post_meta(1, 'geniewords_variant_status')[0];
                    if ($variant_option === '1') {
                        $variant_option = 'on';
                    } else {
                        $variant_option = 'off';
                    }
                }

                $attachment_ids = $wc_product->get_gallery_image_ids();

                $images = array();

                foreach( $attachment_ids as $attachment_id ) {
                    array_push( $images, wp_get_attachment_url($attachment_id));
                }

                if (isset($variant_option) && $variant_option == 'on') {
                    if ($wc_product->get_attributes()) {
                        if (sizeof($wc_product->get_attributes()) >= 1) {
                            $args = array(
                                'post_type' => 'product_variation',
                                'post_status' => array('private', 'publish'),
                                'numberposts' => -1,
                                'orderby' => 'menu_order',
                                'order' => 'asc',
                                'post_parent' => get_the_ID() // get parent post-ID
                            );
                            $variations = get_posts($args);
                            $price = 0;
                            $sales_price = null;

                            foreach ($variations as $variation) {
                                // get variation ID
                                $variation_ID = $variation->ID;
                                $product_variation = new WC_Product_Variation($variation_ID);
                                if (!empty($product_variation->regular_price)) {
                                    $price = $product_variation->regular_price;
                                } else if (!empty($product_variation->price)) {
                                    $price = $product_variation->price;
                                }

                                if (!empty($wc_product->get_sale_price())) {
                                    $sales_price = (int)$wc_product->get_sale_price();
                                }

                                $name = implode(" - ", $product_variation->get_variation_attributes());

                                $title = get_the_title() . ' - ' . $name;

                                $sizes = $product_variation->get_attribute('size');
                                $color = $product_variation->get_attribute('color');
                                //$get_version_name[] = explode(',', $product_variation->attribute_summary);

                                if(empty($shortDescription) && empty($longDescription)) {
                                    $description = $title;
                                }

                                $productInfo = array(
                                    "kind" => "content#product",
                                    "offerId" => $variation_ID,
                                    "itemGroupId" => $id,
                                    "title" => $title,
                                    "link" => $link,
                                    "imageLink" => $image_link,
                                    "contentLanguage" => $get_country[0],
                                    "targetCountry" => $get_country[1],
                                    "channel" => "online",
                                    "ageGroup" => "adult",
                                    "availability" => $availability,
                                    "googleProductCategory" => $brand,
                                    "color" => $color,
                                    "condition" => $condition,
                                    "mpn" => $gtin,
                                    "price" => array(
                                        "value" => $price,
                                        "currency" => $woo_currency
                                    ),
                                    "sizes" => [
                                        $sizes
                                    ],
                                    "images" => $images
                                );

                                if($description) {
                                    $productInfo['description'] = $description;
                                }

                                if($sales_price) {
                                    $productInfo['salePrice'] = array(
                                        "value" => $sales_price,
                                        "currency" => $woo_currency
                                    );
                                }

                                $products[] = array(
                                    $productInfo
                                );
                            }
                        }
                    } else {
                        $title = get_the_title();
                        $productAttr = $wc_product->get_default_attributes();
                        $sizes = null;
                        $color = null;
                        $sales_price = null;
                        if (is_array($productAttr) && count($productAttr) > 0) {
                            $color = isset($productAttr['pa_color']) ? $productAttr['pa_color'] : null;
                            $sizes = isset($productAttr['size']) ? $productAttr['size'] : null;
                        }

                        if (!empty($wc_product->sale_price)) {
                            $sales_price = (int)$wc_product->sale_price;
                        }

                        if (!empty($wc_product->regular_price)) {
                            $price = $wc_product->regular_price;
                        } else if (!empty($wc_product->price)) {
                            $price = $wc_product->price;
                        }

                        unset($product_type);
                        $getTags = get_the_terms($id, 'product_cat');
                        $n = 1;
                        $product_type = '';
                        foreach ($getTags as $tag) {
                            $product_type .= $tag->name;
                            $n++;
                            if ($n > 0) {
                                $product_type .= " > ";
                            }
                        }
                        $product_type = substr($product_type, 0, -3);

                        if(empty($shortDescription) && empty($longDescription)) {
                            $description = $title;
                        }

                        $productInfo = array(
                            "kind" => "content#product",
                            "offerId" => $id,
                            "title" => $title,
                            "link" => $link,
                            "imageLink" => $image_link,
                            "contentLanguage" => $get_country[0],
                            "targetCountry" => $get_country[1],
                            "channel" => "online",
                            "ageGroup" => "adult",
                            "availability" => $availability,
                            "googleProductCategory" => $brand,
                            "color" => $color,
                            "condition" => $condition,
                            "mpn" => $gtin,
                            "price" => array(
                                "value" => $price,
                                "currency" => $woo_currency
                            ),
                            "sizes" => [
                                $sizes
                            ],
                            "images" => $images
                        );

                        if($description) {
                            $productInfo['description'] = $description;
                        }

                        if($sales_price) {
                            $productInfo['salePrice'] = array(
                                "value" => $sales_price,
                                "currency" => $woo_currency
                            );
                        }

                        $products[] = array(
                            $productInfo
                        );
                    }
                } else {
                    $title = get_the_title();
                    $productAttr = $wc_product->get_default_attributes();
                    $sizes = null;
                    $color = null;
                    $sales_price = null;
                    $price = 0;
                    if (is_array($productAttr) && count($productAttr) > 0) {
                        $color = isset($productAttr['pa_color']) ? $productAttr['pa_color'] : null;
                        $sizes = isset($productAttr['size']) ? $productAttr['size'] : null;
                    }

                    if (!empty($wc_product->sale_price)) {
                        $sales_price = (int)$wc_product->sale_price;
                    }

                    if (!empty($wc_product->regular_price)) {
                        $price = $wc_product->regular_price;
                    } else if (!empty($wc_product->price)) {
                        $price = $wc_product->price;
                    }

                    unset($product_type);
                    $getTags = get_the_terms($id, 'product_cat');
                    $n = 1;
                    $product_type = '';
                    foreach ($getTags as $tag) {
                        $product_type .= $tag->name;
                        $n++;
                        if ($n > 0) {
                            $product_type .= " > ";
                        }
                    }
                    $product_type = substr($product_type, 0, -3);

                    $productInfo = array(
                        "kind" => "content#product",
                        "offerId" => $id,
                        "title" => $title,
                        "link" => $link,
                        "imageLink" => $image_link,
                        "contentLanguage" => $get_country[0],
                        "targetCountry" => $get_country[1],
                        "channel" => "online",
                        "ageGroup" => "adult",
                        "availability" => $availability,
                        "googleProductCategory" => $brand,
                        "color" => $color,
                        "condition" => $condition,
                        "mpn" => $gtin,
                        "price" => array(
                            "value" => $price,
                            "currency" => $woo_currency
                        ),
                        "sizes" => [
                            $sizes
                        ]
                    );

                    if($description) {
                        $productInfo['description'] = $description;
                    }

                    if($sales_price) {
                        $productInfo['salePrice'] = array(
                            "value" => $sales_price,
                            "currency" => $woo_currency
                        );
                    }

                    $products[] = array(
                        $productInfo
                    );
                }
            endwhile;
            return array($products);
        }

        protected function deletePluginInfoFromDb()
        {
            delete_post_meta(1, 'geniewords_userid');
            delete_post_meta(1, 'geniewords_refresh_token');
            delete_post_meta(1, 'geniewords_merchantid');
            delete_post_meta(1, 'geniewords_user_company');
            delete_post_meta(1, 'geniewords_ajax_manually_status');
            delete_post_meta(1, 'geniewords_status');
            delete_post_meta(1, 'geniewords_google_shopping_status');
            delete_post_meta(1, 'geniewords_user_fb_pixel_detect');
            delete_post_meta(1, 'geniewords_facebook_shop_status');
            delete_post_meta(1, 'geniewords_fb_pixelinfo_code');
            delete_post_meta(1, 'geniewords_fb_user_available');
            delete_post_meta(1, 'geniewords_user_loggedin');
            delete_post_meta(1, 'geniewords_gs_user_available');
            delete_post_meta(1, 'geniewords_variant_status');
            delete_post_meta(1, 'geniewords_fb_variant_status');
            delete_post_meta(1, 'geniewords_country_name');
            delete_post_meta(1, 'pixel_custom_value');
            delete_post_meta(1, 'add_facebook_pixel_in_head');
            delete_post_meta(1, 'analytics_custom_value');
            delete_post_meta(1, 'add_analytics_in_head');
            delete_post_meta(1, 'new_version_available');
            delete_post_meta(1, 'geniewords_google_shopping_meta_tag');
            delete_post_meta(1, 'geniewords_facebook_meta_tag');

            set_transient("geniewordsTrans", []);
            set_transient("geniewordsTransFacebook", []);
            set_transient("geniewordsTransGoogle", []);
        }

        /**
         * @param $errorMsg
         */
        protected function setTransientErrorMsg($errorMsg)
        {
            if (isset($_POST) && isset($_POST['name']) && $_POST['name'] == 'ajax') {
                echo '<p style="color:red">' . $errorMsg . '</p>';
                exit;
            }
            set_transient('error_results', $errorMsg, 60);
        }

        protected function setGenieTrans($transMsg)
        {
            $genieTrans = get_transient('geniewordsTrans');
            if (false !== $genieTrans) {
                $genieTrans = array_merge($genieTrans, $transMsg);
            } else {
                $genieTrans = $transMsg;
            }
            set_transient('geniewordsTrans', $genieTrans);
        }

        protected function setGenieTransFacebook($transMsg)
        {
            $genieTrans = get_transient('geniewordsTransFacebook');
            if (false !== $genieTrans) {
                $genieTrans = array_merge($genieTrans, $transMsg);
            } else {
                $genieTrans = $transMsg;
            }
            set_transient('geniewordsTransFacebook', $genieTrans);
        }

        protected function setGenieTransGoogle($transMsg)
        {
            $genieTrans = get_transient('geniewordsTransGoogle');
            if (false !== $genieTrans) {
                $genieTrans = array_merge($genieTrans, $transMsg);
            } else {
                $genieTrans = $transMsg;
            }
            set_transient('geniewordsTransGoogle', $genieTrans);
        }
    }
}
Geniewords_js::init();